# Контекст

В ходе взаимодействия цифрового помощника с пользователем
исполнение [сценария](https://github.com/asbelon/digital-assistant-script) в каждом моменте находится на 
определенном акте. Нахождение исполнения сценария на определенном акте назовем состоянием исполнения сценария. 
Состояние исполнения сценария хранит исполнитель сценария для каждого отдельного пользователя. Состояние 
определяется текущим актом и контекстом. Под контекстом понимается совокупность значений переменных сценария.

Переменные сценария бывают пользовательские и зарезервированные. Пользовательские переменные используются в шаблонах
сообщений, условных выражениях, вариантах выбора и при присвоении значения переменной.

## Имя переменной

Имя пользовательской переменной имеет специальную структуру и ограничения. Имя переменной может состоять из букв (в 
кодировке utf-8), цифр, символа нижнего подчеркивания и символов квадратных скобок, которые допустимо использовать 
только особым образом (см ниже Нотация квадратных скобок).

```
Переменная
```

Переменные принимают значения стокового типа данных. Приведение к другим типам происходит при интерпретации значений 
в условных актах и в интеграции с другими системами через вызов удаленных процедур и REST методов.

За счет нотации квадратных скобок переменные можно организовать в структуры типа ключ-значение (список).

### Нотация квадратных скобок

После основного имени переменной в квадратных скобках указывается имя свойства переменной. Например,
```
Переменная[Свойство]
```
При этом ограничение на имя свойства такое же как и для имени простой переменной.

Вложенность свойств может быть произвольной. Например, 
```
Переменная[Свойство][ЕщеОдноСвойство]
```
Если две и более переменных имеют одинаковую часть имени до закрывающейся квадратной скобки, то эти переменные можно
использовать как список. Например, если заданы две переменные
```
Переменная[Свойство][ОдноСвойство]
```
и
```
Переменная[Свойство][ДругоеСвойство]
```
то можно использовать список
```
$Переменная[Свойство]
```
как переменную в итерациях

## Использование библиотеки

Назначение библиотеки переводить структуру переменных из нотации сценария цифрового помощника в нотацию языка python 
для манипуляции контекстом при исполнении сценария. Для этого используется метод set объекта класса Context.

```python
import Context

vars_list = [{'name': 'Услуги[0][Имя]', 'value': 'Консультация'},
            {'name': 'Услуги[0][Стоимость]', 'value': '500 р'}]
context = Context()
for v in vars_list:
    context.set(v.get("name"), v.get("value"))
```

Инициализация контекста, например, из файла

```python
import json
import Context

with open('./context.json', encoding='utf-8') as f:
    context = Context(json.load(f))
```

Наследование контекста

